/***************************************************************************
 *   Copyright (C) 2004 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <klocale.h>
#include <kglobal.h>
#include <kcharsets.h>
#include <qtextcodec.h>

#include "kencodedlogsource.h"

KEncodedLogSource::KEncodedLogSource(){
}


KEncodedLogSource::KEncodedLogSource(QString name, QColor textColor, bool bold, bool underline, bool italic): KLogSource(name, textColor, bold, underline, italic)
{
}


KEncodedLogSource::~KEncodedLogSource()
{
}


void KEncodedLogSource::insertMenuItems(QPopupMenu* popup) {
    KLogSource::insertMenuItems(popup);
    
    QPopupMenu* encodings = new QPopupMenu(0L, i18n("Encoding"));
    QStringList e = KGlobal::charsets()->descriptiveEncodingNames();
    int index=0;
    for ( QStringList::Iterator it = e.begin(); it != e.end(); it++, index++ ) {
        QString en = (*it);
        int id = encodings->insertItem(en);
        encodings->setItemChecked(id, encoding==en);
        encodings->setItemParameter(id, index);
    }
    popup->insertItem(i18n("Encoding"), encodings);
    connect(encodings, SIGNAL(activated(int)), this, SLOT(encodingsMenuActivated(int)));
}

void KEncodedLogSource::readConfig(KConfig* config) {
    KLogSource::readConfig(config);
    encoding = config->readEntryUntranslated("Encoding");
}

void KEncodedLogSource::writeConfig(KConfig* config) {
    KLogSource::writeConfig(config);
    config->writeEntry("Encoding", encoding);
}

/*!
    \fn KEncodedLogSource::decode(char*, int)
 */
QString KEncodedLogSource::decode(const char* b, int len) {
    QString res;
    
    QTextCodec* textCodec;
    if (encoding==QString::null)
        textCodec=QTextCodec::codecForLocale();
    else {
        KCharsets* charsets = KGlobal::charsets();
        bool ok;
        textCodec = charsets->codecForName(charsets->encodingForName(encoding), ok);
        if (!ok || textCodec==NULL)
            textCodec=QTextCodec::codecForLocale();
    }
    if (textCodec==NULL) //to be on the safe side
        res = QString::fromLocal8Bit(b, len);
    res = textCodec->toUnicode(b, len);
    return res.stripWhiteSpace().replace("<","&lt;").replace(">","&gt;");
}

/*!
    \fn KEncodedLogSource::encodingsMenuActivated(int id)
 */
void KEncodedLogSource::encodingsMenuActivated(int id) {
    if (strcmp(sender()->className(),"QPopupMenu"))
        return;
    QPopupMenu* m = (QPopupMenu*)sender();
    int idx = m->itemParameter(id);

    QStringList e = KGlobal::charsets()->descriptiveEncodingNames();
    encoding=e[idx];
}

#include "kencodedlogsource.moc"
