/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qtextcodec.h>
#include <klocale.h>
#include <kglobal.h>
#include <kcharsets.h>
#include <string.h>
#include <errno.h>
#include "kfilelogsource.h"

KFileLogSource::KFileLogSource(QString fileName ,QColor textColor, bool bold, bool underline, bool italic): KEncodedLogSource(i18n("file:")+fileName, textColor, bold, underline, italic), file(0), timer(0), bufSize(1024) {
    file = new QFile(fileName);
    init();
}

KFileLogSource::~KFileLogSource() {
    if (file)
        delete file;
    if (timer)
        delete timer;
}

void KFileLogSource::init() {
    timer = new QTimer(this);
    timer->start(0);
    connect(timer, SIGNAL(timeout()), SLOT(timeout()));
    open();
}

void KFileLogSource::writeConfig(KConfig* config) {
    KEncodedLogSource::writeConfig(config);
    if (file)
        config->writeEntry("FileName", file->name());
    config->writeEntry("BufSize", bufSize);
}

void KFileLogSource::readConfig(KConfig* config) {
    KEncodedLogSource::readConfig(config);
    file = new QFile(config->readEntryUntranslated("FileName"));
    bufSize = config->readNumEntry("BufSize");
    init();
}

/*!
    \fn KFileLogSource::open()
 */
void KFileLogSource::open() {
    if (file->isOpen())
        return;
    file->open(IO_ReadOnly);
    if (file->isOpen())
        file->at(file->size());
    else { //try again in a minute
        QTimer::singleShot (6000, this, SLOT(open()));
        sendLineAdded(this, i18n("Can't open %1: %2").arg(file->name()).arg(strerror(errno)));
    }
}

bool KFileLogSource::readMore() {
    if( !file->isOpen() )
        return FALSE;

    QString line, text;
    int len;

    // read lines, check them individually, but pass as much as can
    // be read to the log viewer to avoid slow scrolling
    do {
        len = file->readLine(line, bufSize);
        if(len>0 && !line.isEmpty() && line!="\n" && line!="\r\n") {
            sendLineAdded(this, decode((const char*)line, len));
            text += line;
        }

    } while(len > 0);

    if(!text.isEmpty())
        return TRUE;

    return FALSE;
}

/*!
    \fn KFileLogSource::reopen()
 */
void KFileLogSource::reopen()
{
    file->close();
    open();
}

void KFileLogSource::timeout() {
    if (readMore())
        timer->changeInterval(0);
    else
        timer->changeInterval(1000);
}

void KFileLogSource::insertMenuItems(QPopupMenu* popup) {
    KEncodedLogSource::insertMenuItems(popup);    
    popup->insertSeparator();
    int id = popup->insertItem(i18n("Reopen"), this, SLOT(reopen()));
    popup->setItemEnabled(id, file->isOpen());
}

#include "kfilelogsource.moc"
