/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef KFILELOGSOURCE_H
#define KFILELOGSOURCE_H

#include <kconfig.h>
#include <kencodedlogsource.h>
#include <qfile.h>
#include <qtimer.h>

/**
file log source. Periodically checks specified file for new lines and sends them to the log widget.

@author Konstantin Sobolev
*/
class KFileLogSource : public KEncodedLogSource {
    Q_OBJECT
    friend class KLogSource;
public:
    KFileLogSource(QString fileName, QColor textColor = QColor("black"), bool bold=FALSE, bool underline=FALSE, bool italic=FALSE);

    ~KFileLogSource();

    int getBufSize() {
        return bufSize;
    };
    void setBufSize(int bufSize) {
        this->bufSize=bufSize;
    };

    virtual void insertMenuItems(QPopupMenu* popup);
    virtual void writeConfig(KConfig* config);
    virtual void readConfig(KConfig* config);
protected:
    KFileLogSource(){};

    QFile* file;
    QTimer* timer;
    int bufSize;

    virtual KLogSource::LogSourceType getType() {return KLogSource::File;};
    bool readMore();
protected slots:
    void open();
    void reopen();
    void timeout();
private:
    void init();
};

#endif
