/***************************************************************************
 *   Copyright (C) 2004 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <klocale.h>
#include "kfilter.h"

KFilter::KFilter() {
    enabled = TRUE;
    alert = FALSE;
    //expression = "";
    //comment = "";
}

KFilter::~KFilter() {}

bool KFilter::exclude(QString line) {
    if (!enabled)
        return false;

    if (action==KFilter::AlertOnly)
        return false;

    return expression.exactMatch(line)^(action==KFilter::Include);
}

bool KFilter::doAlert(QString line) {
    if (!enabled)
        return false;

    return alert && expression.exactMatch(line)^(action==KFilter::Include);
}

void KFilter::writeConfig(KConfig* config) {
    config->writeEntry("Enabled", enabled);
    config->writeEntry("StopIfMatches", stopIfMatches);
    config->writeEntry("Expression", expression.pattern());
    config->writeEntry("Action", action);
    config->writeEntry("Alert", alert);
    config->writeEntry("AlertMessage", alertMessage);
    config->writeEntry("Comment", comment);
}

void KFilter::readConfig(KConfig* config) {
    enabled = config->readBoolEntry("Enabled");
    stopIfMatches = config->readBoolEntry("StopIfMatches");
    expression = QRegExp(config->readEntry("Expression"));
    action = (KFilter::KFilterAction)config->readNumEntry("Action", KFilter::Exclude);
    alert = config->readBoolEntry("Alert");
    alertMessage = config->readEntry("AlertMessage");
    comment = config->readEntry("Comment");
}

QStringList KFilter::getActionNames() {
    QStringList actionNames;
    actionNames<<i18n("Exclude matching")<<i18n("Include only matching")<<i18n("Alert only");
    return actionNames;
}
