/***************************************************************************
 *   Copyright (C) 2004 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FILTER_H
#define FILTER_H

#include <kconfig.h>
#include <qstring.h>
#include <qregexp.h>
#include <qstringlist.h>

/**
@author Konstantin Sobolev
*/
class KFilter{
public:
    enum KFilterAction {Exclude, Include, AlertOnly};
    
    KFilter();
    virtual ~KFilter();
    
    void setEnabled(bool enabled) {this->enabled = enabled;};
    bool isEnabled() {return enabled;};
    
    void setStopIfMatches(bool stopIfMatches) {this->stopIfMatches = stopIfMatches;};
    bool isStopIfMatches() {return stopIfMatches;};
    
    void setExpression(QString expression) {this->expression = QRegExp(expression);};
    QRegExp getExpression() {return expression;};
    
    void setAction(KFilterAction action) {this->action = action;};
    KFilterAction getAction() {return action;};
    
    void setAlert(bool alert) {this->alert = alert;};
    bool isAlert() {return alert;};
    
    void setAlertMessage(QString alertMessage) {this->alertMessage = alertMessage;};
    QString getAlertMessage() {return alertMessage;};
    
    void setComment(QString comment) {this->comment = comment;};
    QString getComment() {return comment;};
    
    virtual bool exclude(QString);
    virtual bool doAlert(QString);
    
    virtual void writeConfig(KConfig*);
    virtual void readConfig(KConfig*);
    
    static QStringList getActionNames();
protected:    
    bool enabled;
    bool stopIfMatches;
    QRegExp expression;
    KFilterAction action;
    bool alert;
    QString alertMessage;
    QString comment;
};

#endif
