/***************************************************************************
 *   Copyright (C) 2004 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qtable.h>
#include <qstringlist.h>
#include "kfilterscollection.h"
#include "kfilterseditordlg.h"

KFiltersCollection::KFiltersCollection() {
    filters.setAutoDelete(true);
}

KFiltersCollection::~KFiltersCollection() {}

bool KFiltersCollection::passes(QString line) {
    KFilter* f;
    for(f=filters.first(); f; f=filters.next())
        if (f->exclude(line))
            return false;
        else if (f->getAction()==KFilter::Include && f->isStopIfMatches())
            return true;
    return true;
}

QString KFiltersCollection::alert(QString line) {
    KFilter* f;
    for(f=filters.first(); f; f=filters.next())
        if (f->doAlert(line))
            return f->getAlertMessage();
    return QString::null;
}

void KFiltersCollection::readConfig(KConfig* config, QString logSourceName) {
    int nFilters = config->readNumEntry("NumberOfFilters");
    editorGeometry = config->readRectEntry("FiltersEditorGeometry");
    filters.clear();
    for (int i=1; i<=nFilters; i++) {
        config->setGroup(logSourceName+": filter "+QString::number(i));
        KFilter* f = new KFilter();
        f->readConfig(config);
        filters.append(f);
    }
}

void KFiltersCollection::writeConfig(KConfig* config, QString logSourceName) {
    config->writeEntry("NumberOfFilters", filters.count());
    config->writeEntry("FiltersEditorGeometry", editorGeometry);
    for (unsigned int i=1; i<=filters.count(); i++) {
        config->setGroup(logSourceName+": filter "+QString::number(i));
        filters.at(i-1)->writeConfig(config);
    }
}

void KFiltersCollection::edit(QWidget* parent) {
    KFiltersEditorDlg dlg(parent);

    //fill table
    int row=0;
    KFilter* f;
    for(f=filters.first(); f;f=filters.next(), row++){
        dlg.dataTable->insertRows(row);

        QCheckTableItem* isEnabledItem = new QCheckTableItem(dlg.dataTable, "");
        isEnabledItem->setChecked(f->isEnabled());
        dlg.dataTable->setItem(row, 0, isEnabledItem);
        
        QCheckTableItem* isStopItem = new QCheckTableItem(dlg.dataTable, "");
        if (f->getAction() == KFilter::Include) {
            isStopItem->setChecked(f->isStopIfMatches());
            isStopItem->setEnabled(TRUE);
        } else {
            isStopItem->setChecked(TRUE);
            isStopItem->setEnabled(FALSE);
        }
        dlg.dataTable->setItem(row, 1, isStopItem);

        dlg.dataTable->setItem(row, 2, new QTableItem(
            dlg.dataTable, QTableItem::WhenCurrent,
            f->getExpression().pattern()
        ));

        QComboTableItem* actionItem = new QComboTableItem(dlg.dataTable, KFilter::getActionNames(), FALSE);
        actionItem->setCurrentItem(f->getAction());
        dlg.dataTable->setItem(row, 3, actionItem);

        QCheckTableItem* isAlertItem = new QCheckTableItem(dlg.dataTable, "");
        isAlertItem->setChecked(f->isAlert());
        dlg.dataTable->setItem(row, 4, isAlertItem);

        QTableItem* alertMessageItem = new QTableItem(
            dlg.dataTable, QTableItem::WhenCurrent,
            f->getAlertMessage()
        );
        alertMessageItem->setEnabled(f->isAlert());
        dlg.dataTable->setItem(row, 5, alertMessageItem);

        dlg.dataTable->setItem(row, 6, new QTableItem(
            dlg.dataTable, QTableItem::WhenCurrent,
            f->getComment()
        ));
    }

    if (editorGeometry.isValid())
        dlg.setGeometry(editorGeometry);

    //run dialog
    //dlg.show();
    dlg.exec();
    editorGeometry=dlg.geometry();

    if (dlg.result()==QDialog::Rejected)
        return;

    //fill with new data
    filters.clear();
    for (row=0; row<dlg.dataTable->numRows(); row++) {
        f = new KFilter();

        QCheckTableItem* isEnabledItem = (QCheckTableItem*) dlg.dataTable->item(row, 0);
        f->setEnabled(isEnabledItem->isChecked());

        QCheckTableItem* isStopItem = (QCheckTableItem*) dlg.dataTable->item(row, 1);
        f->setStopIfMatches(isStopItem->isChecked());
        
        f->setExpression(dlg.dataTable->text(row, 2));

        QComboTableItem* actionItem = (QComboTableItem*) dlg.dataTable->item(row, 3);
        f->setAction((KFilter::KFilterAction)actionItem->currentItem());

        QCheckTableItem* isAlertItem = (QCheckTableItem*) dlg.dataTable->item(row, 4);
        f->setAlert(isAlertItem->isChecked());

        f->setAlertMessage(dlg.dataTable->text(row, 5));
        f->setComment(dlg.dataTable->text(row, 6));

        filters.append(f);
    }
}
