/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/

#include <kiconloader.h>
#include <kglobal.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qstringlist.h>
#include <kparts/part.h>
#include <kparts/componentfactory.h>
#include <kregexpeditorinterface.h>

#include "kfilter.h"

static bool adjusting;
static bool autoAdjust;

void KFiltersEditorDlg::updateButtons() {
    bool selectionIsValid = dataTable->currentSelection()!=-1;
    if (selectionIsValid) {
        QTableSelection selection = dataTable->selection(dataTable->currentSelection());
        int from = selection.topRow();
        int to = selection.bottomRow();
        
        selectionIsValid = (from!=-1 && to!=-1 && to>=from);
    }
    
    bool regexpEditorAvailable = KParts::ComponentFactory::createInstanceFromQuery<QDialog>("KRegExpEditor/KRegExpEditor")!=0;
    
    deleteButton->setEnabled(selectionIsValid);
    editRegexpButton->setEnabled(selectionIsValid && regexpEditorAvailable);        
}

void KFiltersEditorDlg::insertNewRow() {
    int newRowIndex = dataTable->numRows();
    dataTable->insertRows(newRowIndex);
    
    QCheckTableItem* isEnabledItem = new QCheckTableItem(dataTable, "");
    isEnabledItem->setChecked(true);
    dataTable->setItem(newRowIndex, 0, isEnabledItem);
    
    QCheckTableItem* isStopItem = new QCheckTableItem(dataTable, "");
    isStopItem->setEnabled(FALSE);
    isStopItem->setChecked(TRUE);
    dataTable->setItem(newRowIndex, 1, isStopItem);
    
    dataTable->setItem(newRowIndex, 2, new QTableItem(dataTable, QTableItem::WhenCurrent));
    dataTable->setItem(newRowIndex, 3, new QComboTableItem(dataTable, KFilter::getActionNames(), FALSE)); 
    dataTable->setItem(newRowIndex, 4, new QCheckTableItem(dataTable, ""));
    
    QTableItem* alertMessageItem = new QTableItem(dataTable, QTableItem::WhenCurrent);
    alertMessageItem->setEnabled(false);
    dataTable->setItem(newRowIndex, 5, alertMessageItem);
    
    dataTable->setItem(newRowIndex, 6, new QTableItem(dataTable, QTableItem::WhenCurrent));            
 
    dataTable->selectRow(newRowIndex);
    Q_ASSERT(dataTable->currentSelection()!=-1);
    updateButtons();
    adjustColumns();    
}

void KFiltersEditorDlg::editCurrentRowRegexp() {
    int selectionNum = dataTable->currentSelection();
    if (selectionNum == -1) return;
    
    int selectedRow = dataTable->selection(selectionNum).anchorRow();
    QTableItem* item = dataTable->item(selectedRow, 2);
    
    if (!item) return;
    
    QDialog *editorDialog = KParts::ComponentFactory::createInstanceFromQuery<QDialog>( "KRegExpEditor/KRegExpEditor");
    if ( editorDialog ) {
        editorDialog->setIcon(KGlobal::iconLoader()->loadIcon("klogview", KIcon::Small));
        
        // kdeutils was installed, so the dialog was found fetch the editor interface
        KRegExpEditorInterface *editor = static_cast<KRegExpEditorInterface *>( editorDialog->qt_cast( "KRegExpEditorInterface" ));
        Q_ASSERT( editor ); // This should not fail!

        // now use the editor.
        editor->setRegExp(item->text());

        // Finally exec the dialog
        editorDialog->exec();

        if (editorDialog->result()==QDialog::Accepted) {
            item->setText(editor->regExp());
            dataTable->updateCell(selectedRow, 3);
            adjustColumns();
        }
    } /*else
        item->setText(KInputDialog::getText(title, title, defaultValue, status, NULL));*/
        
    //dataTable->updateCell(selectedRow, 3);
}


void KFiltersEditorDlg::deleteRows() {
    int selectionNum = dataTable->currentSelection();
    if (selectionNum == -1) return;
    
    QTableSelection selection = dataTable->selection(selectionNum);
    int from = selection.topRow();
    int to = selection.bottomRow();
    
    for(int row=to; row>=from; row--)
        dataTable->removeRow(row);
        
    updateButtons();
}

void KFiltersEditorDlg::init() {
    setIcon(KGlobal::iconLoader()->loadIcon("klogview", KIcon::Small));

    //disable left header
    //dataTable->setLeftMargin(0);
    //dataTable->verticalHeader()->hide();
    
    //dataTable->setColumnStretchable(0, TRUE); //enabled
    //dataTable->setColumnStretchable(1, TRUE); //regexp
    //dataTable->setColumnStretchable(2, TRUE); //action    
    //dataTable->setColumnStretchable(4, TRUE); //alert message
    dataTable->setColumnStretchable(6, TRUE); //comment
    
    adjusting=FALSE;
    autoAdjust=TRUE;
    
    connect(dataTable->horizontalHeader(), SIGNAL(sizeChange(int, int, int)), this, SLOT(columnWidthChanged()));    
        
    QTimer::singleShot(200, this, SLOT(postInit()));
}

void KFiltersEditorDlg::postInit() {
    updateButtons();
    adjustColumns();
}

void KFiltersEditorDlg::valueChanged(int row, int col) {
    QCheckTableItem* isStopItem = (QCheckTableItem*) dataTable->item(row, 1);
    QComboTableItem* actionItem = (QComboTableItem*) dataTable->item(row, 3);
    QCheckTableItem* alertItem = (QCheckTableItem*) dataTable->item(row, 4);
    QTableItem* alertMessageItem = dataTable->item(row, 5);

    if(col==3) {
        int currentItem = actionItem->currentItem();
        if (currentItem == (int)KFilter::AlertOnly) {
            alertItem->setChecked(true);
            alertItem->setEnabled(false);
            alertMessageItem->setEnabled(true);            
        } else
            alertItem->setEnabled(true);                        
                
        if (currentItem == (int)KFilter::Exclude)
            isStopItem->setChecked(TRUE);
        isStopItem->setEnabled(currentItem != (int)KFilter::Exclude);
    } else if(col==4) {
        alertMessageItem->setEnabled(alertItem->isChecked());    
    }
    
    adjustColumns();
}

void KFiltersEditorDlg::adjustColumns() {
    if (!adjusting && autoAdjust) {
        adjusting = TRUE;
        for (int i=0; i<dataTable->numCols(); dataTable->adjustColumn(i++));
        adjusting = FALSE;
    }
}

void KFiltersEditorDlg::columnWidthChanged() {
    autoAdjust = FALSE;
}
