/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef KLOGSOURCE_H
#define KLOGSOURCE_H

#include <qobject.h>
#include <qstring.h>
#include <qcolor.h>
#include <qpopupmenu.h>
#include <kconfig.h>

#include "kfilterscollection.h"

/**
abstract log source

@author Konstantin Sobolev
*/
class KLogSource : public QObject {
    Q_OBJECT

public:
    KLogSource(QString name, QColor textColor = QColor("black"), bool bold=FALSE, bool underline=FALSE, bool italic=FALSE);

    void setTextColor(QColor textColor) {
        this->textColor=textColor;
        emit styleChanged(this);
    };
    void setBold(bool bold) {
        this->bold=bold;
        emit styleChanged(this);
    };
    void setUnderline(bool underline) {
        this->underline=underline;
        emit styleChanged(this);
    };
    void setItalic(bool italic) {
        this->italic=italic;
        emit styleChanged(this);
    };
    void setEnabled(bool enabled) {
        this->enabled=enabled;
    }

    QString getName() {
        return name;
    };
    QColor getTextColor() {
        return textColor;
    };
    bool isBold() {
        return bold;
    };
    bool isUnderline() {
        return underline;
    };
    bool isItalic() {
        return italic;
    };
    bool isEnabled() {
        return enabled;
    }

    ~KLogSource();
    virtual void writeConfig(KConfig* config);
    virtual void readConfig(KConfig* config);
    virtual void insertMenuItems(QPopupMenu* popup);
    static KLogSource* createFromConfig(KConfig* config);
protected:
    KLogSource() {};
    enum LogSourceType {File, Process};

    QString name;
    QColor textColor;
    bool bold;
    bool underline;
    bool italic;

    KFiltersCollection filters;
    bool enabled;

    virtual KLogSource::LogSourceType getType() = 0;
    virtual void sendLineAdded(KLogSource* source, const QString& line);
    void resetDontShowAlertAgain();
    
    //why why why QT has so ugly events model?!
    //last popup menu
    QPopupMenu* popup;
    //menu item IDs    
    int enabledId;
    int boldId;
    int italicId;
    int underlineId;
private:
    QString askForRegExp(QString title, QString defaultValue, bool* status);
    bool showingAlert;
signals:
    void lineAdded(KLogSource* source, const QString& line);
    void styleChanged(KLogSource* source);
protected slots:    
    void changeBold();
    void changeColor();
    void changeItalic();
    void changeUnderline();
    void changeFilters();
    void changeEnabled();
};

#endif
