/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "klogtray.h"
#include <qpixmap.h>
#include <kapp.h>
#include <kglobal.h>
#include <kkeydialog.h>
#include <kiconloader.h>
#include <kaboutapplication.h>

KLogTray::KLogTray(KLogView* parent, const char* name): KSystemTray(parent, name), parent(parent) {
    init();
}

void KLogTray::init() {
    KIconLoader &loader= *KGlobal::iconLoader();
    QPixmap pixmap = loader.loadIcon("klogview", KIcon::Small);
    setPixmap(pixmap);

    globalAccel = new KGlobalAccel(this);
    globalAccel->insert("ToggleState", i18n("Toggle State"), i18n("Toggles the state of KLogView"), 0, 0,
                        this, SLOT(minimizeRestoreLogView()));
    globalAccel->readSettings(kapp->config());
    globalAccel->updateConnections();

    QPixmap hotkeyPixmap = loader.loadIcon("key_bindings", KIcon::Small);
    contextMenu()->insertItem(hotkeyPixmap, i18n("Set hotkey"), this, SLOT(toggleHotKey()));
    contextMenu()->insertSeparator();
    QPixmap aboutPixmap = loader.loadIcon("viewmag", KIcon::Small);
    contextMenu()->insertItem(aboutPixmap, i18n("About klogview"), this, SLOT(showAboutDialog()));
}

KLogTray::~KLogTray() {}

void KLogTray::minimizeRestoreLogView() {
    KActionCollection* ac = actionCollection();
    KAction* ma = ac->action("minimizeRestore");
    if (ma!=NULL)
        ma->activate();
}

void KLogTray::toggleHotKey() {
    KKeyDialog::configure(globalAccel);
    globalAccel->updateConnections();
    globalAccel->writeSettings(kapp->config());
}

void KLogTray::showAboutDialog() {
    KAboutApplication* dlg = new KAboutApplication(parent->getAboutData(), parent);
    dlg->show();
}

#include "klogtray.moc"
