/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef KLOGWIDGET_H
#define KLOGWIDGET_H

#include <kconfig.h>
#include <ktextedit.h>
#include <kfiledialog.h>
#include <qptrlist.h>
#include <qdict.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qfont.h>
#include <qmutex.h>

#include "klogsource.h"
#include "klogview.h"

class KLogView;

/**
@author Konstantin Sobolev
*/
class KLogWidget : public KTextEdit {
    Q_OBJECT

public:
    KLogWidget(QWidget* parent=0, const char* name=0);
    ~KLogWidget();

    void addLogSource(KLogSource* source);
    void removeLogSource(KLogSource* source);
    void setLogView (KLogView* logView) {
        this->logView=logView;
    };
    void writeConig(KConfig* config);
    void readConfig(KConfig* config);

    void insertMenuItems (QPopupMenu* popup);
protected slots:
    void appendLogLine(KLogSource* source,const QString& line);
    void updateStyleTag(KLogSource* source);
    void addFileLogSource();
    void addProcessOutputLogSource();
protected:
    QPtrList<KLogSource> logSources;
    QDict<QString> tags;
    KLogView* logView;
    QFont font;
    QMutex appendLogLineMutex;
    int deleteItemId;

    QPopupMenu* createPopupMenu(const QPoint& pos);
    QPopupMenu* buildLogSourceMenu(KLogSource* source, int index);
/*private:
    static int counter;*/
private slots:
    void menuActivated (int id);
    void changeFont();
};

#endif
