/***************************************************************************
 *   Copyright (C) 2003 by Konstantin Sobolev                              *
 *   konstantin.sobolev@gmail.com                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include <klocale.h>
#include <kinputdialog.h>
#include "kprocessoutputlogsource.h"


KProcessOutputLogSource::KProcessOutputLogSource(QString command, bool autorun, QColor textColor, bool bold, bool underline, bool italic)
        : KEncodedLogSource(i18n("process:")+command, textColor, bold, underline, italic), command(command), autorun(autorun) {

    initProcess();
    setCommand(command, autorun);
}

KProcessOutputLogSource::~KProcessOutputLogSource() {}

void KProcessOutputLogSource::initProcess() {
    process.setUseShell(TRUE); //?
    connect(&process,SIGNAL(processExited(KProcess*)), this, SLOT(processExited(KProcess*)));
    connect(&process,SIGNAL(receivedStdout(KProcess*, char*, int)), SLOT(receivedStdout(KProcess*, char*, int)));
    connect(&process,SIGNAL(receivedStderr(KProcess*, char*, int)), SLOT(receivedStderr(KProcess*, char*, int)));
}

/*!
    \fn KProcessOutputLogSource::setCommand(QString command, bool start)
 */
void KProcessOutputLogSource::setCommand(QString command, bool start) {
    this->command=command;
    //name=i18n("process:")+command; //will break tags
    //parse?
    stop();
    process.clearArguments();
    process<<command; //parse?? what to do with redirection and piping?
    if (start)
        this->start();
}

/*!
    \fn KProcessOutputLogSource::processExited(KProcess*)
 */
void KProcessOutputLogSource::processExited(KProcess* p) {
    sendLineAdded(this, i18n("Process exited with status %1").arg(p->exitStatus()));
}

#define UNUSED(x) (void)x

/*!
    \fn KProcessOutputLogSource::receivedStdout(KProcess*, char*, int)
 */
void KProcessOutputLogSource::receivedStdout(KProcess* p, char* b, int l) {
    UNUSED(p);
    QStringList lines = QStringList::split(QRegExp("\\n"), decode(b, l), TRUE);
    for ( QStringList::Iterator it = lines.begin(); it != lines.end(); ++it ) {
        sendLineAdded(this, *it);
    }
}

/*!
    \fn KProcessOutputLogSource::receivedStderr(KProcess*, char*, int)
 */
void KProcessOutputLogSource::receivedStderr(KProcess* p, char* b, int l) {
    UNUSED(p);
    sendLineAdded(this, QString("[err]")+decode(b, l)); //todo color?
}

void KProcessOutputLogSource::writeConfig(KConfig* config) {
    KEncodedLogSource::writeConfig(config);
    config->writeEntry("Command", command);
    config->writeEntry("Autorun", autorun);
}

void KProcessOutputLogSource::readConfig(KConfig* config) {
    KEncodedLogSource::readConfig(config);
    command = config->readEntryUntranslated("Command");
    autorun = config->readBoolEntry("Autorun");
    initProcess();
    setCommand(command, autorun);
}

void KProcessOutputLogSource::insertMenuItems(QPopupMenu* popup) {
    KEncodedLogSource::insertMenuItems(popup);    
    popup->insertSeparator();
    //popup->insertItem(i18n("Set command"), this, SLOT(setCommand()));
    int id = popup->insertItem(i18n("Start"), this, SLOT(start()));
    popup->setItemEnabled(id, !process.isRunning());
    id = popup->insertItem(i18n("Stop"), this, SLOT(stop()));
    popup->setItemEnabled(id, process.isRunning());
    id = popup->insertItem(i18n("Autorun"), this, SLOT(toggleAutorun()));
    popup->setItemChecked(id, autorun);
}

/*!
    \fn KProcessOutputLogSource::start()
 */
void KProcessOutputLogSource::start() {
    resetDontShowAlertAgain();
    if (!process.start(KProcess::NotifyOnExit, KProcess::AllOutput))
        sendLineAdded(this, i18n("Error starting process"));
}


/*!
    \fn KProcessOutputLogSource::stop()
 */
void KProcessOutputLogSource::stop() {
    if (process.isRunning())
        if (!process.kill()) {
            sendLineAdded(this, i18n("{Process doesn't wan't to die, using SIGKILL}"));
            if (!process.kill(SIGKILL))
                sendLineAdded(this, i18n("{Process doesn't wan't to die with SIGKILL! giving up}"));
        }
}


/*!
    \fn KProcessOutputLogSource::setCommand()
 */
void KProcessOutputLogSource::setCommand() {
    bool ok;
    QString cmd = KInputDialog::getText(i18n("Command"), i18n("Command: ") ,command, &ok, NULL);
    if (!ok)
        return;
    setCommand(cmd, autorun);
}

/*!
    \fn KProcessOutputLogSource::setAutorun()
 */
void KProcessOutputLogSource::toggleAutorun() {
    autorun=!autorun;
}

#include "kprocessoutputlogsource.moc"
